;***************************************************************************
;**                                                                        *
;**  FILE        :  cstart.asm                                             *
;**                                                                        *
;**  DESCRIPTION :  C startup code                                         *
;**                                                                        *
;**  NOTE        :  When using EDE all macros are automatically set by the *
;**                 Project, Cpu and Bus configuration... dialog from the  *
;**                 EDE menu.                                              *
;**                                                                        *
;**  Copyright 1996-2002 Altium BV                                         *
;**                                                                        *
;**************************************************************************/

        .global _START                  ; reset vector address
        .global _exit                   ; exit return address

        .extern main                    ; start label user C program

.if !@DEF('_NO_EXIT')
        .extern exit                    ; C library exit function
.endif
.if !@DEF('_NO_USP_INIT')
        .extern _lc_ue_ustack           ; user stack end
.endif
.if !@DEF('_NO_ISP_INIT')
        .extern _lc_ue_istack           ; interrupt stack end
.endif
.if !@DEF('_NO_BIV_INIT')
        .extern (CODE) _lc_u_int_tab    ; interrupt table
.endif
.if !@DEF('_NO_BTV_INIT')
        .extern (CODE) _lc_u_trap_tab   ; trap table
.endif
.if !@DEF('_NO_A0A1_ADDRESSING')
        .extern (DATA) _lc_gb_a0        ; start of A0 addressable area
        .extern (DATA) _lc_gb_a1        ; start of A1 addressable area
.endif
.if !@DEF('_NO_A8A9_ADDRESSING')
        .extern (DATA) _lc_gb_a8        ; start of A8 addressable area
        .extern (DATA) _lc_gb_a9        ; start of A9 addressable area
.endif
.if !@DEF('_NO_C_INIT')
        .extern _c_init                 ; C initialization function
.endif  
.if @DEF('_SYNC_ON_HALT')
        .extern _sync_on_halt           ; CrossView utilty function
.endif  

;
;       _REG<cpu_name>_DEF macro's are defined in the reg<cpu_name>.def SFR
;       system include files. These files can be included by the assembler 
;       with the -C option. (e.g. -Ctc10gp) If non of these macro's
;       is defined the default regtc10gp.def SFR file is included. Based on 
;       the selected CPU in EDE the derivative specific SFR file is 
;       automatically included.
;
.if !( @DEF('_REGTC10GP_DEF') || @DEF('_REGPXB4260_DEF') || @DEF('_REGTC1775A_DEF') || @DEF('_REGTC1775B_DEF') || @DEF('_REGTC11IB_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1912_DEF') || @DEF('_REGTC1920A_DEF') ||  @DEF('_REGTC1920B_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGRAPTOR_DEF') || @DEF('_REGTC1765_DEF') || @DEF('_REGRIDERD_DEF') || @DEF('_REGTC2_DEF') || @DEF('_REGUSER112_DEF') || @DEF('_REGUSER113_DEF') )
.include <regtc10gp.def>                ; include default SFR's
.endif

;
;       Deviating naming conventions
;
.if @DEF('_REGTC11IB_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGRIDERD_DEF')
        WDT_CON0        .equ    WDTCON0
        WDT_CON1        .equ    WDTCON1
.endif
.if @DEF('_REGTC11IB_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1912_DEF') || @DEF('_REGTC1920A_DEF') || @DEF('_REGTC1920B_DEF')
        EBU_ADDSEL0     .equ    EBU_ADDRSEL0
        EBU_ADDSEL1     .equ    EBU_ADDRSEL1
        EBU_ADDSEL2     .equ    EBU_ADDRSEL2
        EBU_ADDSEL3     .equ    EBU_ADDRSEL3
.endif  
.if @DEF('_REGTC1920A_DEF')
        PMU_CON0        .equ    PMU_CON
.endif  
.if @DEF('_REGRAPTOR_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1912_DEF') ||  @DEF('_REGTC1920B_DEF')
        S_BCU_CON       .equ    BCU_CON
.endif
.if @DEF('_REGTC1920A_DEF')
        S_BCU_CON       .equ    BCUCON
.endif
.if @DEF('_REGTC11IB_DEF')
        S_BCU_CON       .equ    BCU1_CON
.endif
.if @DEF('_REGTC11IB_DEF')
        F_BCU_CON       .equ    BCU0_CON
.endif

;       macro for loading 32bit constants into address registers
CONST.A .MACRO  ax,v
        movh.a  ax,#@his(v)             ; high part with correction for signed addition
        lea     ax,[ax]@los(v)
        .ENDM

;       macro for loading 32bit constants into data registers
CONST.D .MACRO  dx,v
        mov.u   dx,#@lo(v)
        addih   dx,dx,#@hi(v)
        .ENDM

;       macro for 32bit indirect call
CALL.I  .MACRO  ax,v
        movh.a  ax,#@his(v)             ; high part with correction for signed addition
        lea     ax,[ax]@los(v)          ; call function indirectly to have more
.if @DEF('_TC113_CPU16') || @DEF('_CPU_TC048') 
        nop        
.endif                
        calli   ax                      ; freedom with CODE section allocation
        .ENDM   


;       Reset "vector"
;       no .org: placed via the locator in appropriate boot section (see file tri.i)
;
        .sdecl  "reset",CODE
        .sect   "reset"
_START:
.include "function_entry.asm_functional_problem_workaround"

;
;       Turn TC113_CPU14 CPU functional defect checking off
;
.if @DEF('_TC113_CPU14')
$TC113_CPU14    OFF
.endif  
        ja      _start

;
;       BOOTCFG
;       External Boot Memory Configuration Word
;       (Boot memory Offset Address + 0x4)
;
.if     !(@DEF('_BOOTCFG_AGEN'))
_BOOTCFG_AGEN           .equ    0x0     ; demuxed device
.endif
.if     !(@DEF('_BOOTCFG_WAITRDC'))
_BOOTCFG_WAITRDC        .equ    0x0C    ; 3*cmult read wait states
.endif
.if     !(@DEF('_BOOTCFG_ADDRC'))
_BOOTCFG_ADDRC          .equ    0x0     ; no setup cycle (TC113 only)
.endif
.if     !(@DEF('_BOOTCFG_WAIT'))
_BOOTCFG_WAIT           .equ    0x0     ; no wait state insertion
.endif
.if     !(@DEF('_BOOTCFG_SETUP'))
_BOOTCFG_SETUP          .equ    0x0     ; no setup cycle (TC112 only)
.endif
.if     !(@DEF('_BOOTCFG_WAITINV'))
_BOOTCFG_WAITINV        .equ    0x0     ; wait input is low active
.endif
.if     !(@DEF('_BOOTCFG_BCGEN'))
_BOOTCFG_BCGEN          .equ    0x0     ; chip select mode
.endif
.if     !(@DEF('_BOOTCFG_CMULT'))
_BOOTCFG_CMULT          .equ    0x0     ; waitrdc*1 wait states
.endif
.if     !(@DEF('_BOOTCFG_CFG'))
_BOOTCFG_CFG            .equ    0x8080  ; 32bit device
.endif
        .word   ( _BOOTCFG_AGEN | _BOOTCFG_WAITRDC | _BOOTCFG_ADDRC | _BOOTCFG_WAIT | _BOOTCFG_SETUP | _BOOTCFG_WAITINV | _BOOTCFG_BCGEN | _BOOTCFG_CMULT | _BOOTCFG_CFG )

;       Startup code at reset, residing in actual program space
;       no .org: placed via the locator (see file tri.i)
;
        .sdecl  "code.cstart",CODE,ABS24
        .sect   "code.cstart"
        .align  4
_start:
.include "function_entry.asm_functional_problem_workaround"

;
;       Re-enable and reset the call depth counter and  make A0,A1,A8,A9 
;       write-able. It is required for CrossView that these RESET values are
;       restored for each time the startup code is executed.
;
.if !@DEF('_NO_PSW_RESET')
        mfcr    d0,#PSW
        andn    d0,d0,#0x7f             ; reset counter
        insert  d0,d0,#1,#7,#1          ; enable
        insert  d0,d0,#1,#8,#1          ; set GW bit
        mtcr    #PSW,d0
        isync
.endif

;
;       TC112_COR16 workaround:
;               The stack pointers are aligned to quad-word boundary
;               to workaround the TC112_COR16 functional problem.
;               Also C compiler workaround for this CPU function problem
;               need to be enabled with -zTC112_COR16, to align circular
;               buffers on a quad-word boundary and to size all stack 
;               frames to an integral number of quad-words.
;
.if @DEF('_TC112_COR16')
        STACK_ALIGN     .equ    0xf
.else
        STACK_ALIGN     .equ    0x7
.endif

;
;       Load user stack pointer
;
;       Disable this if not started from RESET vector. (E.g.
;       ROM monitors require to keep in control of vectors)
;
.if !@DEF('_NO_USP_INIT')
        CONST.A sp,(_lc_ue_ustack&~STACK_ALIGN) ; initialize user stack pointer
.endif

;
;       Clear Previous Context Pointer Segment Address and Offset Field.
;       It is required for CrossView stack trace that these RESET values 
;       are restored for each time the startup code is executed.
;
.if !@DEF('_NO_PCX_RESET')
        mfcr    d0,#PCXI
        movh    d1,#0xfff0
        and16   d0,d1
        mtcr    #PCXI,d0
        isync
.endif

;
;       Setup the context save area lists
;
;       Tables with start/end addresses go in a separate 'csa_areas' section
;
.if !@DEF('_NO_CSA_INIT')
;
        .sdecl  "csa_areas", CODE
        .sect   "csa_areas"
        .align  4

.if @DEF('_CPU_TC051')
        MAX_NR_OF_CSA_AREAS     .equ    3
.else
        MAX_NR_OF_CSA_AREAS     .equ    1
.endif

csa_area_begin:                         ; csa_area_begin[MAX_NR_OF_CSA_AREAS]
.if @DEF('_CPU_TC051')
        .extern _lc_ub_csa.01           ; context save area 1 begin
        .extern _lc_ub_csa.02           ; context save area 2 begin
        .extern _lc_ub_csa.03           ; context save area 3 begin
; 
        .word   _lc_ub_csa.01
        .word   _lc_ub_csa.02
        .word   _lc_ub_csa.03
.else
        .extern _lc_ub_csa              ; context save area begin
; 
        .word   _lc_ub_csa
.endif

csa_area_end:                           ; csa_area_end[MAX_NR_OF_CSA_AREAS]
.if @DEF('_CPU_TC051')
        .extern _lc_ue_csa.01           ; context save area 1 end
        .extern _lc_ue_csa.02           ; context save area 2 end
        .extern _lc_ue_csa.03           ; context save area 3 end
; 
        .word   _lc_ue_csa.01   
        .word   _lc_ue_csa.02
        .word   _lc_ue_csa.03
.else
        .extern _lc_ue_csa              ; context save area end
; 
        .word   _lc_ue_csa
.endif
;
;        Resume with the original 'cstart' section 
; 
        .sect   "code.cstart"                           
;
        mov     d0,#-1                   ; D0 = index of CSA area, refered to as 'i'     
        j       test_max_csa_areas
;
start_linked_list:
        CONST.A a2,csa_area_begin       ; first calculate nr. of CSAs in this area:
        addsc.a a2,a2,d0,#2             ; get csa_area_begin[i]
        CONST.A a3,csa_area_end
        addsc.a a3,a3,d0,#2             ; get csa_area_end[i]
        ld16.w  d15,[a3]                ; load csa_area_end[i] address
        ld16.w  d2,[a2]                 ; load csa_area_begin[i] address
        sub     d15,d2                  ; 
        sh      d15,#-6                 ; (csa_area_begin-csa_area_end)/64
        jz      d15,test_max_csa_areas  ; if no CSA in this area, then get next area
;
        ld16.a  a2,[a2]                 ; A2 = csa_area_begin[i] address
        mov16.d d1,a2                   ;               
        extr.u  d1,d1,#28,#4            ; extract segment number
        sh      d1,d1,#16               ; D1 = shifted segment number
        jnz     d0,next_areas                   
;
first_area:                             ; Initialize the first time only:
        mov16.d d3,a2                   ; D3 = first CSA from first area 
        mov16   d2,#0
        st.w    [a2+]64,d2              ; store null pointer in (last!) CSA
        add16   d15,#-1                 ; one CSA handled
;
        mov16.d d2,a2                   ; 
        extr.u  d2,d2,#6,#16            ; get CSA index
        or16    d2,d1                   ; add segment number
        mtcr    #LCX,d2                 ; initialize LCX (second CSA in first area)
        isync
        j       load_loop_count
;
next_areas:                             ; D3 = last CSA from previous area
        extr.u  d7,d3,#28,#4            ; extract segment number
        sh      d7,d7,#16               ; D1 = shifted segment number
        extr.u  d2,d3,#6,#16            ; get CSA index
        or16    d2,d7                   ; add segment number
        mov16.d d3,a2                   ; D3 = first CSA from this area
        st.w    [a2+]64,d2              ; store in first CSA from this area
        add16   d15,#-1                 ; one CSA handled
;
load_loop_count:
        jz      d15,init_fcx            ; if just one CSA in this area, then ready
        add16   d15,#-1                 ; 'correct' loopcount
        mov.a   a5,d15                  ; A5 = loop counter
loop:                                   ; D3 = previous CSA
        extr.u  d2,d3,#6,#16            ; get CSA index
        or16    d2,d1                   ; add segment number
        mov16.d d3,a2
        st.w    [a2+]64,d2              ; store "next" pointer
;
.include 'loop_before.asm_functional_problem_workaround'
;
        loop16  a5,loop
;
init_fcx:
        extr.u  d2,d3,#6,#16            ; get CSA index
        or16    d2,d1                   ; add segment number
        mtcr    #FCX,d2                 ; initialize FCX
        isync
;
test_max_csa_areas:
        add16   d0,#1                   ; increment index
        jlt     d0,#MAX_NR_OF_CSA_AREAS,start_linked_list
.endif
;
;
; Clear the ENDINIT bit in the WDT_CON0 register in order
; to disable the write-protection for registers protected 
; via the EndInit feature (for example: WDT_CON1).
;
        CALL.I  a2,_endinit_clear
;
;
; Call a function (assembly label) before the write-protection
; is enabled again. In this routine one can initialize the 
; registers protected via the EndInit feature.
; Beware that protected registers are unlocked for the duration 
; of the Time-out Period only!
;
.if @DEF('_CALL_ENDINIT')
        .extern _CALL_ENDINIT
        CALL.I  a2,_CALL_ENDINIT
.endif 
;
; Disable the Watchdog if requested. 
;       
.if @DEF('_WATCHDOG_DISABLE')
        CONST.A a2,WDT_CON1             ; get address WDT_CON1
        ld16.w  d0,[a2]                 ; load WDT_CON1
        CONST.D d1,0x08                 ; WDTDR is bit 3 of WDT_CON1
        or      d0,d0,d1                ; set WDT_CON1_WDTR 
        st16.w  [a2],d0                 ; request to disable the watchdog
.endif

;
;       TC113_PMU3 workaround:
;               The TLB-A and TLB-B mappings are set to a page size
;               of 16KB to workaround the TC113_PMU3 functional problem.
;
;               MMU_CON reset value: 0x7e0
;
.if @DEF('_TC113_PMU3') && !@DEF('_REGRAPTOR_DEF')
        mov     d15,#(0x7e0 & ~0x1e | 0x14)
        mtcr    #MMU_CON,d15
.endif

;
;       Load Base Address of Trap Vector Table
;
;       Disable this if not started from RESET vector. (E.g.
;       ROM monitors require to keep in control of vectors)
;
.if !@DEF('_NO_BTV_INIT')
        CONST.D d0,_lc_u_trap_tab
        mtcr    #BTV,d0
        isync
.endif

;
;       Load Base Address of Interrupt Vector Table
;
;       Disable this if not started from RESET vector. (E.g.
;       ROM monitors require to keep in control of vectors)
;
.if !@DEF('_NO_BIV_INIT')
        CONST.D d0,_lc_u_int_tab
        mtcr    #BIV,d0
        isync
.endif

;
;       Load interupt stack pointer
;
;       Disable this if not started from RESET vector. (E.g.
;       ROM monitors require to keep in control of vectors)
;
.if !@DEF('_NO_ISP_INIT')
        CONST.D d0,(_lc_ue_istack&~STACK_ALIGN)
        mtcr    #ISP,d0                         ; initialize interrupt stack pointer
        isync
.endif

.if @DEF('_CUSTOM_BOARD')
.if @DEF('_REGTC1775A_DEF')
;
;       TC1775A PMU config (Reset 3F06)
;
        CONST.D d2, ( _PMUCON_CDIS | _PMUCON_CBP | _PMUCON_DCLS | _PMUCON_TIRF | _PMUCON_TREF | _PMUCON_FLACC | _PMUCON_IFWC | _PMUCON_SEWC | _PMUCON_PFLWER )
        CONST.A a5, PMU_CON
        st.w [a5], d2

.elif @DEF('_REGTC1775B_DEF') || @DEF('_REGTC1765_DEF') 
;
;       TC1775B and TC1765 PMU config (Reset 3F06)
;       
        CONST.D d2, ( _PMUCON_CDIS | _PMUCON_CBP | _PMUCON_TREF | _PMUCON_FLACC | _PMUCON_FLRFB | _PMUCON_SEWC | 0x04000000 )
        CONST.A a5, PMU_CON
        st.w [a5], d2

.elif @DEF('_REGTC11IB_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1920B_DEF')
;
;       TC11IB, TC1910 and TC1920B PMU config (Reset 3F06)
;       
        CONST.D d2, (_PMUCON0_CCBYP)
        CONST.A a5, PMU_CON0
        st.w [a5], d2
.else
;
;       Code cache control (Reset 02H)
;
        CONST.D d2, (_PMUCON0_CCBYP | _PMUCON0_CCSIZ)
        CONST.A a5, PMU_CON0
        st.w [a5], d2
.endif

.if @DEF('_REGTC1775A_DEF') || @DEF('_REGTC1775B_DEF') || @DEF('_REGTC1765_DEF')
;
;       TC1775A/B and TC1765 PMU_EIFCON config (Reset 5F)
;
        CONST.D d2, ( _PMU_EIFCON_ADVLEN | _PMU_EIFCON_RDWLEN | _PMU_EIFCON_DATLEN | _PMU_EIFCON_WAITFUNC | _PMU_EIFCON_FBBMSEL | _PMU_EIFCON_FBBLEN | _PMU_EIFCON_EIFBLEN | _PMU_EIFCON_CS0D | _PMU_EIFCON_SIDC )
        CONST.A a5, PMU_EIFCON
        st.w [a5], d2
.endif
.endif  

;
;       Data cache control (Reset 0H)
;
;
;       TC113_CPU13 workaround:
;               The 16KB D-Cache is enabled to workaround
;               the TC113_CPU13 functional problem.     
;               NOTE:   Setting the D-Cache size must be done at the
;                       very beginning of the startup code.
;
.if @DEF('_TC113_CPU13')        
        mov16   d15,#1
        CONST.A a15, DMU_CON
        st.w    [a15], d15
.elif @DEF('_DMUCON_DCAON')
.if (_DMUCON_DCAON != 0x0)      
        CONST.D d2, (_DMUCON_DCAON)
        CONST.A a5, DMU_CON
        st.w [a5], d2
.endif
.endif

;
;       Bus configuration EBU_CLC within endinit for TC11IB, TC1910, 
;       TC1912, TC1920A, TC1920B, PXB4225, Raptor and User defined 1.3
;
.if @DEF('_CUSTOM_BOARD')

.if @DEF('_REGTC11IB_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1912_DEF') || @DEF('_REGTC1920A_DEF') ||  @DEF('_REGTC1920B_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGRAPTOR_DEF') || @DEF('_REGUSER113_DEF')

        CONST.D d2, _EBUCLC_DISR
        CONST.A a5, EBU_CLC
        st.w    [a5], d2

;
;       Bus configuration EBU_CLC within endinit for TC10GP, TC1775A, 
;       TC1775B, TC1765, Rider-D, PXB4260 and User-defined 1.2
;
.elif @DEF('_REGTC10GP_DEF') || @DEF('_REGPXB4260_DEF') || @DEF('_REGUSER112_DEF') || @DEF('_REGRIDERD_DEF')
        CONST.D d2, (_EBUCLC_DISR | _EBUCLC_EDIS)
        st.w EBU_CLC, d2
.else
        CONST.D d2, (_EBUCLC_DISR)
        st.w EBU_CLC, d2
.endif
.endif

;
; Set the ENDINIT bit in the WDT_CON0 register again
; to enable the write-protection and to prevent a time-out. 
;
        CALL.I  a2,_endinit_set

;
;       TC113_PMU1 workaround:
;               The split mode is disabled on the LMB bus to workaround
;               the TC113_PMU1 functional problem.
;
.if @DEF('_TC113_PMU1')
        mov16   d7,#-2
        CONST.A a15, LFI_CON
        ld.w    d15,[a15]
        and16   d15,d7
        st.w    [a15], d15      
.endif

;
;       TC112_COR15 workaround:
;               The starvation protection is disabled to workaround
;               the TC112_COR15 functional problem.
;
.if @DEF('_TC112_COR15')
        CONST.A a3,BCU_CON
        CONST.D d2,0xFFF7FFFF
        ld16.w  d0,[a3]
        and     d0,d2,d0
        st16.w  [a3],d0 
.endif

;
;       Inititialize global address registers a0/a1 to support
;       _a0/_a1 storage qualifiers of the C compiler.
;
.if !@DEF('_NO_A0A1_ADDRESSING')
        CONST.A a0,_lc_gb_a0+0x8000
        CONST.A a1,_lc_gb_a1+0x8000
.endif

;
;       Inititialize global address registers a8/a9 to support
;       _a8/_a9 storage qualifiers of the C compiler. A8 and A9
;       are reserved for OS use, or for application use in cases 
;       where the application ans OS are tightly coupled.
;
.if !@DEF('_NO_A8A9_ADDRESSING')
        CONST.A a8,_lc_gb_a8+0x8000
        CONST.A a9,_lc_gb_a9+0x8000
.endif

;
;       Initialize Bus configuration registers
;       
.If !@DEF('_NO_BUS_CONF')
;
;       Target board bus configuration
;
;       Set register-values according to .define's created by the EDE
;       if a .define is not present, a default value will be used
;
.if @DEF('_TARGET_BOARD')
        .if @DEF('_EBUCON_VALUE')
                CONST.D d2, _EBUCON_VALUE
                CONST.A a15, EBU_CON
                st.w    [a15], d2
        .else
        ; set default-value for EBUCON
        .endif
        .if @DEF('_BUSCON0_VALUE')
                CONST.D d2, _BUSCON0_VALUE
                CONST.A a15, EBU_BUSCON0
                st.w    [a15], d2
        .else
        ; set default-value for BUSCON0
        .endif
        .if @DEF('_BUSCON1_VALUE')
                CONST.D d2, _BUSCON1_VALUE
                CONST.A a15, EBU_BUSCON1
                st.w    [a15], d2
        .else
        ; set default-value for BUSCON1
        .endif
        .if @DEF('_BUSCON2_VALUE')
                CONST.D d2, _BUSCON2_VALUE
                CONST.A a15, EBU_BUSCON2
                st.w    [a15], d2
        .else
        ; set default-value for BUSCON2
        .endif
        .if @DEF('_BUSCON3_VALUE')
                CONST.D d2, _BUSCON3_VALUE
                CONST.A a15, EBU_BUSCON3
                st.w    [a15], d2
        .else
        ; set default-value for BUSCON3
        .endif
        .if @DEF('_ADDSEL0_VALUE')
                CONST.D d2, _ADDSEL0_VALUE
                CONST.A a15, EBU_ADDSEL0
                st.w    [a15], d2
        .else
        ; set default-value for ADDSEL0
        .endif
        .if @DEF('_ADDSEL1_VALUE')
                CONST.D d2, _ADDSEL1_VALUE
                CONST.A a15, EBU_ADDSEL1
                st.w    [a15], d2
        .else
        ; set default-value for ADDSEL1
        .endif
        .if @DEF('_ADDSEL2_VALUE')
                CONST.D d2, _ADDSEL2_VALUE
                CONST.A a15, EBU_ADDSEL2
                st.w    [a15], d2
        .else
        ; set default-value for ADDSEL2
        .endif
        .if @DEF('_ADDSEL3_VALUE')
                CONST.D d2, _ADDSEL3_VALUE
                CONST.A a15, EBU_ADDSEL3
                st.w    [a15], d2
        .else
        ; set default-value for ADDSEL3
        .endif
        .if @DEF('_DRMCON0_VALUE')
                CONST.D d2, _DRMCON0_VALUE
                CONST.A a15, EBU_DRMCON0
                st.w    [a15], d2
        .else
        ; set default-value for DRMCON0
        .endif
        .if @DEF('_DRMCON1_VALUE')
                CONST.D d2, _DRMCON1_VALUE
                CONST.A a15, EBU_DRMCON1
                st.w    [a15], d2
        .else
        ; set default-value for DRMCON1
        .endif
        .if @DEF('_BUSAP0_VALUE')
                CONST.D d2, _BUSAP0_VALUE
                CONST.A a15, EBU_BUSAP0
                st.w    [a15], d2
        .else
        ; set default-value for BUSAP0
        .endif
        .if @DEF('_BUSAP1_VALUE')
                CONST.D d2, _BUSAP1_VALUE
                CONST.A a15, EBU_BUSAP1
                st.w    [a15], d2
        .else
        ; set default-value for BUSAP1
        .endif
        .if @DEF('_BUSAP2_VALUE')
                CONST.D d2, _BUSAP2_VALUE
                CONST.A a15, EBU_BUSAP2
                st.w    [a15], d2
        .else
        ; set default-value for BUSAP2
        .endif
        .if @DEF('_BFCON_VALUE')
                CONST.D d2, _BFCON_VALUE
                CONST.A a15, EBU_BFCON
                st.w    [a15], d2
        .else
        ; set default-value for BFCON
        .endif
        .if @DEF('_SDRMCON0_VALUE')
                CONST.D d2, _SDRMCON0_VALUE
                CONST.A a15, EBU_SDRMCON0
                st.w    [a15], d2
        .else
        ; set default-value for SDRMCON0
        .endif
        .if @DEF('_SDRMMOD0_VALUE')
                CONST.D d2, _SDRMMOD0_VALUE
                CONST.A a15, EBU_SDRMOD0
                st.w    [a15], d2
        .else
        ; set default-value for SDRMMOD0
        .endif
        .if @DEF('_SDRMREF0_VALUE')
                CONST.D d2, _SDRMREF0_VALUE
                CONST.A a15, EBU_SDRMREF0
                st.w    [a15], d2
        .else
        ; set default-value for SDRMREF0
        .endif
        .if @DEF('_MCDBBS_VALUE')
                CONST.D d2, _MCDBBS_VALUE
                CONST.A a15, MCDBBS 
                st.w    [a15], d2
        .else
        ; set default-value for MCDBBS
        .endif
.else
;
;       User defined target board bus configuration
;
;       Settings for user defined board. Values from editboxes 
;       are shifted to their correct place.
;
.elif @DEF('_CUSTOM_BOARD')

.if @DEF('_REGTC11IB_DEF') || @DEF('_REGTC1910_DEF') || @DEF('_REGTC1912_DEF') || @DEF('_REGTC1920A_DEF') ||  @DEF('_REGTC1920B_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGRAPTOR_DEF') || @DEF('_REGUSER113_DEF')
;
;       Bus configuration for TC11IB, TC1910, TC1912, TC1920A, TC1920B, PXB4225, 
;       Raptor and User defined 1.3
;
;       Naming convention changed for the TC1910/TC1912/TC1920A/TC1920B/Raptor
;       for BCU Control Register
;
        
        CONST.D d2, ((_S_BCUCON_SPC << 24) | _S_BCUCON_SPE | _S_BCUCON_PSE | _S_BCUCON_DBG | _S_BCUCON_TOUT )
        st.w S_BCU_CON, d2


.if @DEF('_REGTC11IB_DEF') || @DEF('_REGPXB4225_DEF') || @DEF('_REGUSER113_DEF')
        CONST.D d2, ((_F_BCUCON_SPC << 24) | _F_BCUCON_SPE | _F_BCUCON_PSE | _F_BCUCON_DBG | _F_BCUCON_PMS | _F_BCUCON_TOUT )
        CONST.A a5, F_BCU_CON
        st.w    [a5], d2
.endif

.if @DEF('_REGTC1910_DEF') || @DEF('_REGTC1920B_DEF')
        CONST.D d2, (_EBUCON_EXTRECON | _EBUCON_EXTSVM | _EBUCON_EXTACC | _EBUCON_EXTLOCK | _EBUCON_ARBSYNC | _EBUCON_ARBMODE | (_EBUCON_TOUTC << 8) | ( _EBUCON_GLOBALCS << 16) | _EBUCON_BUSCLK )
        CONST.A a5, EBU_CON
        st.w    [a5], d2

        CONST.D d2, ( _BFCON_FETBLEN0 | _BFCON_FBBMSEL0 | _BFCON_WAITFUNC0 | _BFCON_EXTCLOCK | _BFCON_BFCMSEL0 | _BFCON_EBSE0 | _BFCON_FETBLEN1 | _BFCON_FBBMSEL1 | _BFCON_WAITFUNC1 | _BFCON_BFCMSEL1 | _BFCON_EBSE1 )
        CONST.A a5, EBU_BFCON
        st.w    [a5], d2
.else
        
        CONST.D d2, (_EBUCON_EXTRECON | _EBUCON_EXTSVM | _EBUCON_EXTACC | _EBUCON_EXTLOCK | _EBUCON_ARBSYNC | _EBUCON_ARBMODE | (_EBUCON_TOUTC << 8) | ( _EBUCON_GLOBALCS << 16) | _EBUCON_BUSCLK )
        CONST.A a5, EBU_CON
        st.w    [a5], d2

        CONST.D d2, ( _BFCON_FETBLEN | _BFCON_FBBMSEL | _BFCON_WAITFUNC | _BFCON_EXTCLOCK )
        CONST.A a5, EBU_BFCON
        st.w    [a5], d2
.endif

.if @DEF('_REGTC1910_DEF') || @DEF('_REGTC1920B_DEF') 
        CONST.D d2, ( _SDRMREF0_REFRESHC | _SDRMREF0_REFRESHR | _SDRMREF0_SELFREX | _SDRMREF0_SELFREN | _SDRMREF0_AUTOSELFR )
        CONST.A a5, EBU_SDRMREF0
        st.w    [a5], d2
.else
        CONST.D d2, ( _SDRMREF0_REFRESHC | _SDRMREF0_REFRESHR | _SDRMREF0_SELFREX | _SDRMREF0_SELFREN )
        CONST.A a5, EBU_SDRMREF0
        st.w    [a5], d2
.endif

        CONST.D d2, ( _SDRMREF1_REFRESHC | _SDRMREF1_REFRESHR )
        CONST.A a5, EBU_SDRMREF1
        st.w    [a5], d2

        CONST.D d2, ( _SDRMCON0_CRAS | _SDRMCON0_CRFSH | _SDRMCON0_CRSC | _SDRMCON0_CRP | _SDRMCON0_AWIDTH | _SDRMCON0_CRCD | _SDRMCON0_CRC | _SDRMCON0_PAGEM | _SDRMCON0_BANKM )
        CONST.A a5, EBU_SDRMCON0
        st.w    [a5], d2

        CONST.D d2, (_SDRMOD0_BURSTL | _SDRMOD0_CASLAT)
        CONST.A a5, EBU_SDRMOD0
        st.w    [a5], d2

        CONST.D d2, (_SDRMOD1_BURSTL | _SDRMOD1_CASLAT)
        CONST.A a5, EBU_SDRMOD1
        st.w    [a5], d2

        CONST.D d2, ( _SDRMCON1_CRAS | _SDRMCON1_CRFSH | _SDRMCON1_CRSC | _SDRMCON1_CRP | _SDRMCON1_AWIDTH | _SDRMCON1_CRCD | _SDRMCON1_CRC | _SDRMCON1_PAGEM | _SDRMCON1_BANKM )
        CONST.A a5, EBU_SDRMCON1
        st.w    [a5], d2

.if @DEF('_REGTC11IB_DEF')
        CONST.D d2, ( _BUSCON0_MULTMAP | _BUSCON0_WPRE | _BUSCON0_AALIGN | _BUSCON0_CMULT | _BUSCON0_ENDIAN | _BUSCON0_DLOAD | _BUSCON0_PRE | _BUSCON0_WAITINV | _BUSCON0_BCGEN | _BUSCON0_PORTW | _BUSCON0_WAIT | _BUSCON0_XCMDDELAY | _BUSCON0_AGEN | _BUSCON0_WR )
        CONST.A a5, EBU_BUSCON0
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON1_MULTMAP | _BUSCON1_WPRE | _BUSCON1_AALIGN | _BUSCON1_CMULT | _BUSCON1_ENDIAN | _BUSCON1_DLOAD | _BUSCON1_PRE | _BUSCON1_WAITINV | _BUSCON1_BCGEN | _BUSCON1_PORTW | _BUSCON1_WAIT | _BUSCON1_XCMDDELAY | _BUSCON1_AGEN | _BUSCON1_WR )
        CONST.A a5, EBU_BUSCON1
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON2_MULTMAP | _BUSCON2_WPRE | _BUSCON2_AALIGN | _BUSCON2_CMULT | _BUSCON2_ENDIAN | _BUSCON2_DLOAD | _BUSCON2_PRE | _BUSCON2_WAITINV | _BUSCON2_BCGEN | _BUSCON2_PORTW | _BUSCON2_WAIT | _BUSCON2_XCMDDELAY | _BUSCON2_AGEN | _BUSCON2_WR )
        CONST.A a5, EBU_BUSCON2
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON3_MULTMAP | _BUSCON3_WPRE | _BUSCON3_AALIGN | _BUSCON3_CMULT | _BUSCON3_ENDIAN | _BUSCON3_DLOAD | _BUSCON3_PRE | _BUSCON3_WAITINV | _BUSCON3_BCGEN | _BUSCON3_PORTW | _BUSCON3_WAIT | _BUSCON3_XCMDDELAY | _BUSCON3_AGEN | _BUSCON3_WR )
        CONST.A a5, EBU_BUSCON3
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON4_MULTMAP | _BUSCON4_WPRE | _BUSCON4_AALIGN | _BUSCON4_CMULT | _BUSCON4_ENDIAN | _BUSCON4_DLOAD | _BUSCON4_PRE | _BUSCON4_WAITINV | _BUSCON4_BCGEN | _BUSCON4_PORTW | _BUSCON4_WAIT | _BUSCON4_XCMDDELAY | _BUSCON4_AGEN | _BUSCON4_WR )
        CONST.A a5, EBU_BUSCON4
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON5_MULTMAP | _BUSCON5_WPRE | _BUSCON5_AALIGN | _BUSCON5_CMULT | _BUSCON5_ENDIAN | _BUSCON5_DLOAD | _BUSCON5_PRE | _BUSCON5_WAITINV | _BUSCON5_BCGEN | _BUSCON5_PORTW | _BUSCON5_WAIT | _BUSCON5_XCMDDELAY | _BUSCON5_AGEN | _BUSCON5_WR )
        CONST.A a5, EBU_BUSCON5
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON6_MULTMAP | _BUSCON6_WPRE | _BUSCON6_AALIGN | _BUSCON6_CMULT | _BUSCON6_ENDIAN | _BUSCON6_DLOAD | _BUSCON6_PRE | _BUSCON6_WAITINV | _BUSCON6_BCGEN | _BUSCON6_PORTW | _BUSCON6_WAIT | _BUSCON6_XCMDDELAY | _BUSCON6_AGEN | _BUSCON6_WR )
        CONST.A a5, EBU_BUSCON6
        st.w    [a5], d2

.elif @DEF('_REGTC1910_DEF') || @DEF('_REGTC1920B_DEF')
        CONST.D d2, ( _BUSCON0_MULTMAP | _BUSCON0_WPRE | _BUSCON0_AALIGN | (_BUSCON0_CTYPE << 10) | _BUSCON0_CMULT | _BUSCON0_ENDIAN | _BUSCON0_DLOAD | _BUSCON0_PRE | _BUSCON0_WAITINV | _BUSCON0_BCGEN | _BUSCON0_PORTW | _BUSCON0_WAIT | _BUSCON0_XCMDDELAY | _BUSCON0_AGEN | _BUSCON0_WR )
        CONST.A a5, EBU_BUSCON0
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON1_MULTMAP | _BUSCON1_WPRE | _BUSCON1_AALIGN | (_BUSCON1_CTYPE << 10) | _BUSCON1_CMULT | _BUSCON1_ENDIAN | _BUSCON1_DLOAD | _BUSCON1_PRE | _BUSCON1_WAITINV | _BUSCON1_BCGEN | _BUSCON1_PORTW | _BUSCON1_WAIT | _BUSCON1_XCMDDELAY | _BUSCON1_AGEN | _BUSCON1_WR )
        CONST.A a5, EBU_BUSCON1
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON2_MULTMAP | _BUSCON2_WPRE | _BUSCON2_AALIGN | (_BUSCON2_CTYPE << 10) | _BUSCON2_CMULT | _BUSCON2_ENDIAN | _BUSCON2_DLOAD | _BUSCON2_PRE | _BUSCON2_WAITINV | _BUSCON2_BCGEN | _BUSCON2_PORTW | _BUSCON2_WAIT | _BUSCON2_XCMDDELAY | _BUSCON2_AGEN | _BUSCON2_WR )
        CONST.A a5, EBU_BUSCON2
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON3_MULTMAP | _BUSCON3_WPRE | _BUSCON3_AALIGN | (_BUSCON3_CTYPE << 10) | _BUSCON3_CMULT | _BUSCON3_ENDIAN | _BUSCON3_DLOAD | _BUSCON3_PRE | _BUSCON3_WAITINV | _BUSCON3_BCGEN | _BUSCON3_PORTW | _BUSCON3_WAIT | _BUSCON3_XCMDDELAY | _BUSCON3_AGEN | _BUSCON3_WR )
        CONST.A a5, EBU_BUSCON3
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON4_MULTMAP | _BUSCON4_WPRE | _BUSCON4_AALIGN | (_BUSCON4_CTYPE << 10) | _BUSCON4_CMULT | _BUSCON4_ENDIAN | _BUSCON4_DLOAD | _BUSCON4_PRE | _BUSCON4_WAITINV | _BUSCON4_BCGEN | _BUSCON4_PORTW | _BUSCON4_WAIT | _BUSCON4_XCMDDELAY | _BUSCON4_AGEN | _BUSCON4_WR )
        CONST.A a5, EBU_BUSCON4
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON5_MULTMAP | _BUSCON5_WPRE | _BUSCON5_AALIGN | (_BUSCON5_CTYPE << 10) | _BUSCON5_CMULT | _BUSCON5_ENDIAN | _BUSCON5_DLOAD | _BUSCON5_PRE | _BUSCON5_WAITINV | _BUSCON5_BCGEN | _BUSCON5_PORTW | _BUSCON5_WAIT | _BUSCON5_XCMDDELAY | _BUSCON5_AGEN | _BUSCON5_WR )
        CONST.A a5, EBU_BUSCON5
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON6_MULTMAP | _BUSCON6_WPRE | _BUSCON6_AALIGN | (_BUSCON6_CTYPE << 10) | _BUSCON6_CMULT | _BUSCON6_ENDIAN | _BUSCON6_DLOAD | _BUSCON6_PRE | _BUSCON6_WAITINV | _BUSCON6_BCGEN | _BUSCON6_PORTW | _BUSCON6_WAIT | _BUSCON6_XCMDDELAY | _BUSCON6_AGEN | _BUSCON6_WR )
        CONST.A a5, EBU_BUSCON6
        st.w    [a5], d2

.else
        CONST.D d2, ( _BUSCON0_MULTMAP | _BUSCON0_WPRE | _BUSCON0_CMULT | _BUSCON0_ENDIAN | _BUSCON0_DLOAD | _BUSCON0_PRE | _BUSCON0_WAITINV | _BUSCON0_BCGEN | _BUSCON0_PORTW | _BUSCON0_WAIT | _BUSCON0_XCMDDELAY | _BUSCON0_AGEN | _BUSCON0_WR )
        CONST.A a5, EBU_BUSCON0
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON1_MULTMAP | _BUSCON1_WPRE | _BUSCON1_CMULT | _BUSCON1_ENDIAN | _BUSCON1_DLOAD | _BUSCON1_PRE | _BUSCON1_WAITINV | _BUSCON1_BCGEN | _BUSCON1_PORTW | _BUSCON1_WAIT | _BUSCON1_XCMDDELAY | _BUSCON1_AGEN | _BUSCON1_WR )
        CONST.A a5, EBU_BUSCON1
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON2_MULTMAP | _BUSCON2_WPRE | _BUSCON2_CMULT | _BUSCON2_ENDIAN | _BUSCON2_DLOAD | _BUSCON2_PRE | _BUSCON2_WAITINV | _BUSCON2_BCGEN | _BUSCON2_PORTW | _BUSCON2_WAIT | _BUSCON2_XCMDDELAY | _BUSCON2_AGEN | _BUSCON2_WR )
        CONST.A a5, EBU_BUSCON2
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON3_MULTMAP | _BUSCON3_WPRE | _BUSCON3_CMULT | _BUSCON3_ENDIAN | _BUSCON3_DLOAD | _BUSCON3_PRE | _BUSCON3_WAITINV | _BUSCON3_BCGEN | _BUSCON3_PORTW | _BUSCON3_WAIT | _BUSCON3_XCMDDELAY | _BUSCON3_AGEN | _BUSCON3_WR )
        CONST.A a5, EBU_BUSCON3
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON4_MULTMAP | _BUSCON4_WPRE | _BUSCON4_CMULT | _BUSCON4_ENDIAN | _BUSCON4_DLOAD | _BUSCON4_PRE | _BUSCON4_WAITINV | _BUSCON4_BCGEN | _BUSCON4_PORTW | _BUSCON4_WAIT | _BUSCON4_XCMDDELAY | _BUSCON4_AGEN | _BUSCON4_WR )
        CONST.A a5, EBU_BUSCON4
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON5_MULTMAP | _BUSCON5_WPRE | _BUSCON5_CMULT | _BUSCON5_ENDIAN | _BUSCON5_DLOAD | _BUSCON5_PRE | _BUSCON5_WAITINV | _BUSCON5_BCGEN | _BUSCON5_PORTW | _BUSCON5_WAIT | _BUSCON5_XCMDDELAY | _BUSCON5_AGEN | _BUSCON5_WR )
        CONST.A a5, EBU_BUSCON5
        st.w    [a5], d2
        CONST.D d2, ( _BUSCON6_MULTMAP | _BUSCON6_WPRE | _BUSCON6_CMULT | _BUSCON6_ENDIAN | _BUSCON6_DLOAD | _BUSCON6_PRE | _BUSCON6_WAITINV | _BUSCON6_BCGEN | _BUSCON6_PORTW | _BUSCON6_WAIT | _BUSCON6_XCMDDELAY | _BUSCON6_AGEN | _BUSCON6_WR )
        CONST.A a5, EBU_BUSCON6
        st.w    [a5], d2
.endif

        CONST.D d2,  ((_ADDRSEL0_BASE << 12) | (_ADDRSEL0_ALTSEG << 8 ) | (_ADDRSEL0_MASK << 4) | _ADDRSEL0_ALTENAB | _ADDRSEL0_REGEN )
        CONST.A a5, EBU_ADDRSEL0
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL1_BASE << 12) | (_ADDRSEL1_ALTSEG << 8 ) | (_ADDRSEL1_MASK << 4) | _ADDRSEL1_ALTENAB | _ADDRSEL1_REGEN )
        CONST.A a5, EBU_ADDRSEL1
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL2_BASE << 12) | (_ADDRSEL2_ALTSEG << 8 ) | (_ADDRSEL2_MASK << 4) | _ADDRSEL2_ALTENAB | _ADDRSEL2_REGEN )
        CONST.A a5, EBU_ADDRSEL2
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL3_BASE << 12) | (_ADDRSEL3_ALTSEG << 8 ) | (_ADDRSEL3_MASK << 4) | _ADDRSEL3_ALTENAB | _ADDRSEL3_REGEN )
        CONST.A a5, EBU_ADDRSEL3
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL4_BASE << 12) | (_ADDRSEL4_ALTSEG << 8 ) | (_ADDRSEL4_MASK << 4) | _ADDRSEL4_ALTENAB | _ADDRSEL4_REGEN )
        CONST.A a5, EBU_ADDRSEL4
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL5_BASE << 12) | (_ADDRSEL5_ALTSEG << 8 ) | (_ADDRSEL5_MASK << 4) | _ADDRSEL5_ALTENAB | _ADDRSEL5_REGEN )
        CONST.A a5, EBU_ADDRSEL5
        st.w    [a5], d2
        CONST.D d2,  ((_ADDRSEL6_BASE << 12) | (_ADDRSEL6_ALTSEG << 8 ) | (_ADDRSEL6_MASK << 4) | _ADDRSEL6_ALTENAB | _ADDRSEL6_REGEN )
        CONST.A a5, EBU_ADDRSEL6
        st.w    [a5], d2

        CONST.D d2,  ( _BUSAP0_DTACS | _BUSAP0_DTARDWR | _BUSAP0_WRRECOVC | _BUSAP0_RDRECOVC | _BUSAP0_DATAC | _BUSAP0_BURSTC | _BUSAP0_WAITWRC | _BUSAP0_WAITRDC | _BUSAP0_CMDDELAY | _BUSAP0_AHOLDC | _BUSAP0_ADDRC )
        CONST.A a5, EBU_BUSAP0
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP1_DTACS | _BUSAP1_DTARDWR | _BUSAP1_WRRECOVC | _BUSAP1_RDRECOVC | _BUSAP1_DATAC | _BUSAP1_BURSTC | _BUSAP1_WAITWRC | _BUSAP1_WAITRDC | _BUSAP1_CMDDELAY | _BUSAP1_AHOLDC | _BUSAP1_ADDRC )
        CONST.A a5, EBU_BUSAP1
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP2_DTACS | _BUSAP2_DTARDWR | _BUSAP2_WRRECOVC | _BUSAP2_RDRECOVC | _BUSAP2_DATAC | _BUSAP2_BURSTC | _BUSAP2_WAITWRC | _BUSAP2_WAITRDC | _BUSAP2_CMDDELAY | _BUSAP2_AHOLDC | _BUSAP2_ADDRC )
        CONST.A a5, EBU_BUSAP2
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP3_DTACS | _BUSAP3_DTARDWR | _BUSAP3_WRRECOVC | _BUSAP3_RDRECOVC | _BUSAP3_DATAC | _BUSAP3_BURSTC | _BUSAP3_WAITWRC | _BUSAP3_WAITRDC | _BUSAP3_CMDDELAY | _BUSAP3_AHOLDC | _BUSAP3_ADDRC )
        CONST.A a5, EBU_BUSAP3
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP4_DTACS | _BUSAP4_DTARDWR | _BUSAP4_WRRECOVC | _BUSAP4_RDRECOVC | _BUSAP4_DATAC | _BUSAP4_BURSTC | _BUSAP4_WAITWRC | _BUSAP4_WAITRDC | _BUSAP4_CMDDELAY | _BUSAP4_AHOLDC | _BUSAP4_ADDRC )
        CONST.A a5, EBU_BUSAP4
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP5_DTACS | _BUSAP5_DTARDWR | _BUSAP5_WRRECOVC | _BUSAP5_RDRECOVC | _BUSAP5_DATAC | _BUSAP5_BURSTC | _BUSAP5_WAITWRC | _BUSAP5_WAITRDC | _BUSAP5_CMDDELAY | _BUSAP5_AHOLDC | _BUSAP5_ADDRC )
        CONST.A a5, EBU_BUSAP5
        st.w    [a5], d2
        CONST.D d2,  ( _BUSAP6_DTACS | _BUSAP6_DTARDWR | _BUSAP6_WRRECOVC | _BUSAP6_RDRECOVC | _BUSAP6_DATAC | _BUSAP6_BURSTC | _BUSAP6_WAITWRC | _BUSAP6_WAITRDC | _BUSAP6_CMDDELAY | _BUSAP6_AHOLDC | _BUSAP6_ADDRC )
        CONST.A a5, EBU_BUSAP6
        st.w    [a5], d2

        CONST.D d2, ((_EMUAS_BASE << 12) | (_EMUAS_ALTSEG << 8) | (_EMUAS_MASK << 4) | _EMUAS_ALTENAB | _EMUAS_REGENAB )
        CONST.A a5, EBU_EMUAS
        st.w    [a5], d2

.if @DEF('_REGTC11IB_DEF')
        CONST.D d2, ( _EMUBC_MULTMAP | _EMUBC_WPRE | _EMUBC_AALIGN | _EMUBC_CMULT | _EMUBC_ENDIAN | _EMUBC_DLOAD | _EMUBC_PRE | _EMUBC_WAITINV | _EMUBC_BCGEN | _EMUBC_PORTW | _EMUBC_WAIT | _EMUBC_XCMDDELAY | _EMUBC_AGEN | _EMUBC_WR )
        CONST.A a5, EBU_EMUBC
        st.w    [a5], d2

.elif @DEF('_REGTC1910_DEF') || @DEF('_REGTC1920B_DEF')
        CONST.D d2, ( _EMUBC_MULTMAP | _EMUBC_WPRE | _EMUBC_AALIGN | _EMUBC_CTYPE | _EMUBC_CMULT | _EMUBC_ENDIAN | _EMUBC_DLOAD | _EMUBC_PRE | _EMUBC_WAITINV | _EMUBC_BCGEN | _EMUBC_PORTW | _EMUBC_WAIT | _EMUBC_XCMDDELAY | _EMUBC_AGEN | _EMUBC_WR )
        CONST.A a5, EBU_EMUBC
        st.w    [a5], d2

.else
        CONST.D d2, ( _EMUBC_MULTMAP | _EMUBC_WPRE | _EMUBC_CMULT | _EMUBC_ENDIAN | _EMUBC_DLOAD | _EMUBC_PRE | _EMUBC_WAITINV | _EMUBC_BCGEN | _EMUBC_PORTW | _EMUBC_WAIT | _EMUBC_XCMDDELAY | _EMUBC_AGEN | _EMUBC_WR )
        CONST.A a5, EBU_EMUBC
        st.w    [a5], d2
.endif

        CONST.D d2,  ( _EMUBAP_DTACS | _EMUBAP_DTARDWR | _EMUBAP_WRRECOVC | _EMUBAP_RDRECOVC | _EMUBAP_DATAC | _EMUBAP_BURSTC | _EMUBAP_WAITWRC | _EMUBAP_WAITRDC | _EMUBAP_CMDDELAY | _EMUBAP_AHOLDC | _EMUBAP_ADDRC )
        CONST.A a5, EBU_EMUBAP
        st.w    [a5], d2

        CONST.D d2, _EMUOVL_OVERLAY
        CONST.A a5, EBU_EMUOVL
        st.w    [a5], d2
        
.else
;
;       Bus configuration for TC10GP, TC1775A, TC1775B, TC1765, 
;       Rider-D, PXB4260 and User-defined 1.2
;

        CONST.D d2, ((_BCUCON_SPC << 24) | _BCUCON_SPE | _BCUCON_PSE | _BCUCON_DBG | _BCUCON_TOUT )
        st.w BCU_CON, d2

.if @DEF('_REGTC1765_DEF')
        CONST.D d2, (_EBUCON_DTACS | _EBUCON_DTARW )
.else
        CONST.D d2, (_EBUCON_DTACS | _EBUCON_DTARW | (_EBUCON_TOUTC << 8)| _EBUCON_ARBMODE | _EBUCON_ARBSYNC | _EBUCON_EXTLOCK | _EBUCON_EXTACC | _EBUCON_EXTSVM | _EBUCON_EXTRECON)
.endif
        st.w EBU_CON, d2

.if @DEF('_REGTC1775A_DEF') || @DEF('_REGTC1775B_DEF')
;
;       TC1775A/B BUSCON0-3 config
;
        CONST.D d2, (_BUSCON0_CMULT | _BUSCON0_RECOVC | _BUSCON0_HOLDC | _BUSCON0_WAITWRC | (_BUSCON0_WAITRDC << 9 ) | _BUSCON0_PORTW | _BUSCON0_SETUP | _BUSCON0_WAITINV | _BUSCON0_WAIT | _BUSCON0_CMULTR | _BUSCON0_AGEN | _BUSCON0_BCGEN | _BUSCON0_ALEC | _BUSCON0_WRDIS )
        st.w EBU_BUSCON0, d2

        CONST.D d2, (_BUSCON1_CMULT | _BUSCON1_RECOVC | _BUSCON1_HOLDC | _BUSCON1_WAITWRC | (_BUSCON1_WAITRDC << 9 ) | _BUSCON1_PORTW | _BUSCON1_SETUP | _BUSCON1_WAITINV | _BUSCON1_WAIT | _BUSCON1_CMULTR | _BUSCON1_AGEN | _BUSCON1_BCGEN | _BUSCON1_ALEC | _BUSCON1_WRDIS )
        st.w EBU_BUSCON1, d2

        CONST.D d2, (_BUSCON2_CMULT | _BUSCON2_RECOVC | _BUSCON2_HOLDC | _BUSCON2_WAITWRC | (_BUSCON2_WAITRDC << 9 ) | _BUSCON2_PORTW | _BUSCON2_SETUP | _BUSCON2_WAITINV | _BUSCON2_WAIT | _BUSCON2_CMULTR | _BUSCON2_AGEN | _BUSCON2_BCGEN | _BUSCON2_ALEC | _BUSCON2_WRDIS )
        st.w EBU_BUSCON2, d2

        CONST.D d2, (_BUSCON3_CMULT | _BUSCON3_RECOVC | _BUSCON3_HOLDC | _BUSCON3_WAITWRC | (_BUSCON3_WAITRDC << 9 ) | _BUSCON3_PORTW | _BUSCON3_SETUP | _BUSCON3_WAITINV | _BUSCON3_WAIT | _BUSCON3_CMULTR | _BUSCON3_AGEN | _BUSCON3_BCGEN | _BUSCON3_ALEC | _BUSCON3_WRDIS )
        st.w EBU_BUSCON3, d2

.elif @DEF('_REGTC1765_DEF')
;
;       TC1765 BUSCON0-3 config
;
        CONST.D d2, (_BUSCON0_CMULT | _BUSCON0_RECOVC | _BUSCON0_HOLDC | _BUSCON0_WAITWRC | (_BUSCON0_WAITRDC << 9 ) | _BUSCON0_PORTW | _BUSCON0_SETUP | _BUSCON0_WAITINV | _BUSCON0_WAIT | _BUSCON0_CMULTR | _BUSCON0_ES | _BUSCON0_BCGEN | _BUSCON0_WRDIS )
        st.w EBU_BUSCON0, d2

        CONST.D d2, (_BUSCON1_CMULT | _BUSCON1_RECOVC | _BUSCON1_HOLDC | _BUSCON1_WAITWRC | (_BUSCON1_WAITRDC << 9 ) | _BUSCON1_PORTW | _BUSCON1_SETUP | _BUSCON1_WAITINV | _BUSCON1_WAIT | _BUSCON1_CMULTR | _BUSCON1_ES | _BUSCON1_BCGEN | _BUSCON1_WRDIS )
        st.w EBU_BUSCON1, d2

        CONST.D d2, (_BUSCON2_CMULT | _BUSCON2_RECOVC | _BUSCON2_HOLDC | _BUSCON2_WAITWRC | (_BUSCON2_WAITRDC << 9 ) | _BUSCON2_PORTW | _BUSCON2_SETUP | _BUSCON2_WAITINV | _BUSCON2_WAIT | _BUSCON2_CMULTR | _BUSCON2_ES | _BUSCON2_BCGEN | _BUSCON2_WRDIS )
        st.w EBU_BUSCON2, d2

        CONST.D d2, (_BUSCON3_CMULT | _BUSCON3_RECOVC | _BUSCON3_HOLDC | _BUSCON3_WAITWRC | (_BUSCON3_WAITRDC << 9 ) | _BUSCON3_PORTW | _BUSCON3_SETUP | _BUSCON3_WAITINV | _BUSCON3_WAIT | _BUSCON3_CMULTR | _BUSCON3_ES | _BUSCON3_BCGEN | _BUSCON3_WRDIS )
        st.w EBU_BUSCON3, d2 
.else
;
;       BUSCON0-3 config 
;
        CONST.D d2, (_BUSCON0_WRDIS | _BUSCON0_AGEN | _BUSCON0_SETUP | _BUSCON0_WAIT | _BUSCON0_WAITINV | _BUSCON0_PORTW | _BUSCON0_ALEC | _BUSCON0_BCGEN | _BUSCON0_WAITWRC | _BUSCON0_WAITRDC | _BUSCON0_HOLDC | _BUSCON0_RECOVC | _BUSCON0_CMULT )
        st.w EBU_BUSCON0, d2

        CONST.D d2, (_BUSCON1_WRDIS | _BUSCON1_AGEN | _BUSCON1_SETUP | _BUSCON1_WAIT | _BUSCON1_WAITINV | _BUSCON1_PORTW | _BUSCON1_ALEC | _BUSCON1_BCGEN | _BUSCON1_WAITWRC | _BUSCON1_WAITRDC | _BUSCON1_HOLDC | _BUSCON1_RECOVC | _BUSCON1_CMULT)
        st.w EBU_BUSCON1, d2

        CONST.D d2, (_BUSCON2_WRDIS | _BUSCON2_AGEN | _BUSCON2_SETUP | _BUSCON2_WAIT | _BUSCON2_WAITINV | _BUSCON2_PORTW | _BUSCON2_ALEC | _BUSCON2_BCGEN | _BUSCON2_WAITWRC | _BUSCON2_WAITRDC | _BUSCON2_HOLDC | _BUSCON2_RECOVC | _BUSCON2_CMULT )
        st.w EBU_BUSCON2, d2

        CONST.D d2, (_BUSCON3_WRDIS | _BUSCON3_AGEN | _BUSCON3_SETUP | _BUSCON3_WAIT | _BUSCON3_WAITINV | _BUSCON3_PORTW | _BUSCON3_ALEC | _BUSCON3_BCGEN | _BUSCON3_WAITWRC | _BUSCON3_WAITRDC | _BUSCON3_HOLDC | _BUSCON3_RECOVC | _BUSCON3_CMULT)
        st.w EBU_BUSCON3, d2
.endif

.if @DEF('_REGRIDERD_DEF')
;
;       RIDER-D ADDSEL0-3 config
;
        CONST.D d2,  ((_ADDSEL0_BASE << 12) | (_ADDSEL0_ALTSEG << 8) | (_ADDSEL0_MASK << 4) | _ADDSEL0_REGEN )
        st.w EBU_ADDSEL0, d2

        CONST.D d2,  ((_ADDSEL1_BASE << 12) | (_ADDSEL1_ALTSEG << 8) | (_ADDSEL1_MASK << 4) | _ADDSEL1_REGEN )
        st.w EBU_ADDSEL1, d2

        CONST.D d2,  ((_ADDSEL2_BASE << 12) | (_ADDSEL2_ALTSEG << 8) | (_ADDSEL2_MASK << 4) | _ADDSEL2_REGEN )
        st.w EBU_ADDSEL2, d2

        CONST.D d2,  ((_ADDSEL3_BASE << 12) | (_ADDSEL3_ALTSEG << 8) | (_ADDSEL3_MASK << 4) | _ADDSEL3_REGEN )
        st.w EBU_ADDSEL3, d2
.else
;
;       ADDSEL0-3 config
;
        CONST.D d2,  ((_ADDSEL0_BASE << 12) | (_ADDSEL0_MASK << 4) | _ADDSEL0_MIRRORE | _ADDSEL0_REGEN )
        st.w EBU_ADDSEL0, d2

        CONST.D d2,  ((_ADDSEL1_BASE << 12) | (_ADDSEL1_MASK << 4) | _ADDSEL1_MIRRORE | _ADDSEL1_REGEN )
        st.w EBU_ADDSEL1, d2

        CONST.D d2,  ((_ADDSEL2_BASE << 12) | (_ADDSEL2_MASK << 4) | _ADDSEL2_MIRRORE | _ADDSEL2_REGEN )
        st.w EBU_ADDSEL2, d2

        CONST.D d2,  ((_ADDSEL3_BASE << 12) | (_ADDSEL3_MASK << 4) | _ADDSEL3_MIRRORE | _ADDSEL3_REGEN )
        st.w EBU_ADDSEL3, d2
.endif

.if @DEF('_REGTC10GP_DEF') || @DEF('_REGPXB4260_DEF') || @DEF('_REGUSER112_DEF') || @DEF('_REGRIDERD_DEF')
;
;       _EBU_DRMCON0-1 config
;
        CONST.D d2,  (_DRMCON0_REFRESHR | _DRMCON0_REFRESHC | _DRMCON0_AWIDTH | _DRMCON0_CRCD | _DRMCON0_CRC | _DRMCON0_CRP | _DRMCON0_CRSC | _DRMCON0_CRFSH | _DRMCON0_CRAS )
        st.w EBU_DRMCON0, d2

        CONST.D d2,  (_DRMCON1_MODE)
        st.w EBU_DRMCON1, d2
.endif

;
;       _EBU_EMUCON config (Reset 0h)
;
        CONST.D d2, (_EBU_EMUCON_OVL0 | _EBU_EMUCON_OVL1 | _EBU_EMUCON_OVL2 | _EBU_EMUCON_OVL3 )
        CONST.A a5, EBU_EMUCON
        st.w [a5], d2

.if @DEF('_REGTC1775B_DEF')
;
;       TC1775B _EBU_EMUBC config (Reset 160280h)
;
        CONST.D d2, (_EBU_EMUBC_CMULT | _EBU_EMUBC_RECOVC | _EBU_EMUBC_HOLDC | _EBU_EMUBC_WAITWRC | (_EBU_EMUBC_WAITRDC << 9 ) | _EBU_EMUBC_PORTW | _EBU_EMUBC_SETUP | _EBU_EMUBC_WAITINV | _EBU_EMUBC_WAIT | _EBU_EMUBC_CMULTR | _EBU_EMUBC_AGEN | _EBU_EMUBC_BCGEN | _EBU_EMUBC_ALEC | _EBU_EMUBC_WRDIS )
        CONST.A a5, EBU_EMUBC
        st.w [a5], d2
.elif @DEF('_REGTC1775A_DEF') || @DEF('_REGRIDERD_DEF')
;
;       TC1775A & RIDER-D _EBU_EMUBC config (Reset 8002D7FFh)
;
        CONST.D d2, (_EBU_EMUBC_WRDIS | _EBU_EMUBC_AGEN | _EBU_EMUBC_SETUP | _EBU_EMUBC_WAIT | _EBU_EMUBC_WAITINV | _EBU_EMUBC_PORTW | _EBU_EMUBC_ALEC | _EBU_EMUBC_BCGEN | _EBU_EMUBC_WAITWRC | _EBU_EMUBC_WAITRDC | _EBU_EMUBC_HOLDC | _EBU_EMUBC_RECOVC | _EBU_EMUBC_CMULT )
        CONST.A a5, EBU_EMUBC
        st.w [a5], d2
.elif @DEF('_REGTC1765_DEF')
;
;       TC1765 _EBU_EMUBC config
;
        CONST.D d2, (_EBU_EMUBC_CMULT | _EBU_EMUBC_RECOVC | _EBU_EMUBC_HOLDC | _EBU_EMUBC_WAITWRC | (_EBU_EMUBC_WAITRDC << 9 ) | _EBU_EMUBC_PORTW | _EBU_EMUBC_SETUP | _EBU_EMUBC_WAITINV | _EBU_EMUBC_WAIT | _EBU_EMUBC_CMULTR | _EBU_EMUBC_ES | _EBU_EMUBC_BCGEN | _EBU_EMUBC_WRDIS )
        CONST.A a5, EBU_EMUBC
        st.w [a5], d2
.endif

.if @DEF('_REGTC1775A_DEF') || @DEF('_REGTC1775B_DEF') || @DEF('_REGRIDERD_DEF') || @DEF('_REGTC1765_DEF')
;
;       TC1775 & RIDER-D _EBU_EMUAS config (Reset BE000031h)
;
        CONST.D d2, (_EBU_EMUAS_REGEN | _EBU_EMUAS_MIRRORE | (_EBU_EMUAS_MASK << 4) | (_EBU_EMUAS_BASE<<12))
        CONST.A a5, EBU_EMUAS
        st.w [a5], d2
.endif  
        
.endif  
.endif
.endif

;
;       Initialize and clear C variables
;
.if !@DEF('_NO_C_INIT')
        CALL.I  a2,_c_init              ; initialize data
.endif  

;
;       Call a function (assembly label) before main()
;
.if @DEF('_CALL_INIT')
        .extern _CALL_INIT
        CALL.I  a2,_CALL_INIT
.endif  


;
;       Clear argc and argv[] of main()
;
.if !@DEF('_NO_ARG_INIT')
        mov16   d4,#0                   ; argc = 0
        mov16.a a4,#0                   ; argv = 0
.endif

;
;       Main default return value is 0
;
.if !@DEF('_NO_EXIT')
        mov16   d2,#0                   ; set default exit value
.endif

;
;       Call C program
;
        CALL.I  a2,main                 ; call main()

;
;       Call C library exit function
;
;       The exit() function causes normal program termination to occur. First, all 
;       functions registered by the atexit() function are called in the reverse 
;       order. Next, all open streams with unwritten buffered data are flushed, all 
;       open streams are closed and all files created by the tmpfile() function are 
;       removed.
;
.if !@DEF('_NO_EXIT')
        mov16   d4,d2                   ; pass return value to exit()
        CALL.I  a2,exit                 ; call exit()
.endif

;
;       The exit() function jumps _exit and a breakpoint is set
;       at _exit by the CrossView debugger to halt execution.
;       
_exit:  debug16                         ; halt the execution if debug mode is enabled  
.include 'loop_before.asm_functional_problem_workaround' 
        loopu   _exit                   ; otherwise loop
;
;       Default trap vectors are resolved from the C-library
;
.if !@DEF('_NO_BTV_INIT')
        .extern (CODE) _trapmmu
        .extern (CODE) _trapprotection
        .extern (CODE) _trapinstruction
        .extern (CODE) _trapcontext
        .extern (CODE) _trapbus
        .extern (CODE) _trapassertion
        .extern (CODE) _trapsystem
        .extern (CODE) _trapnmi
.endif
;
; 
; 
;--------------------------------------------------------------------------
; Utility subroutines
;--------------------------------------------------------------------------
; function:     _endinit_clear
; input:        None
; output:       Bit ENDINIT in WDTCON0 is cleared
; other registers used:
;               D0, D1, D2, D4, A3, A4 - work register
; description:  Clears the ENDINIT bit in the 
;               WDT_CON0 register in order to disable the 
; write-protection for registers protected 
; via the EndInit feature (ie. WDT_CON1, BTV, 
; BIV, ISP, mod_CLC).
;--------------------------------------------------------------------------
;
_endinit_clear:
;
; 1st step: Password access (create password and send to WDT_CON0)
;
        CONST.A a3,WDT_CON0             ; get address WDT_CON0
        ld16.w  d0,[a3]                 ; get WDT_CON0 in d0
;
        CONST.A a4,WDT_CON1             ; get address WDT_CON1
        ld16.w  d1,[a4]                 ; get WDT_CON1_WDTDR WDT_CON1_WDTIR
;
        CONST.D d2,0xFFFFFF01   ; mask hardware password 0,1
        and     d0,d2,d0                ; in copy of WDT_CON0 (d0),
        or      d0,d0,#0xF0             ; set WDTHPW1 to 0xF and load
        and     d4,d1,#0x0C             ; copy of WDT_CON1_WDTDR and
        or      d0,d4,d0                ; WDT_CON1_WDTIR (d1)
        st16.w  [a3],d0                 ; set WDT_CON0
;
; 2nd step: Modify access, set the bit ENDINIT to 0 to allow access to
;                       registers: WDT_CON1, BTV, BIV, ISP and mod_CLC
;
        CONST.D d4,0xFFFFFFF0           ; mask WDTHPW0,WDTLCK,ENDINIT
        and     d0,d4,d0                ; in copy of WDT_CON0 (d0)
        or      d0,d0,#0X02             ; WDTHPW0=0,WDTLCK=1,ENDINIT=0
        isync
        st16.w  [a3],d0                 ; set WDT_CON0
;
        ret                             ;
;
;--------------------------------------------------------------------------
; function:     _endinit_set
; input:
;       None
; output:
;   Bit ENDINIT in WDTCON0 is set
; other registers used:
;       D0, D1, D2, D4, A3, A4 - work register
; description: Clears the ENDINIT bit in the 
; WDT_CON0 register in order to disable the 
; write-protection for registers protected 
; via the EndInit feature (ie. WDT_CON1, BTV, 
; BIV, ISP, mod_CLC).
;--------------------------------------------------------------------------
;
_endinit_set:
;
; 1st step: Password access (create password and send to WDT_CON0)
;
        CONST.A a3,WDT_CON0             ; get address WDT_CON0
        ld16.w  d0,[a3]                 ; get WDT_CON0 in d0
;
        CONST.A a4,WDT_CON1             ; get address WDT_CON1
        ld16.w  d1,[a4]                 ; get WDT_CON1_WDTDR WDT_CON1_WDTIR
;
        CONST.D d2,0xFFFFFF01           ; mask hardware password 0,1
        and     d0,d2,d0                ; in copy of WDT_CON0 (d0),
        or      d0,d0,#0xF0             ; set WDTHPW1 to 0xF and load
        and     d4,d1,#0x0C             ; copy of WDT_CON1_WDTDR and
        or      d0,d4,d0                ; WDT_CON1_WDTIR (d1)
        st16.w  [a3],d0                 ; set WDT_CON0
;
; 2nd step: Modify access, set the bit ENDINIT to 1  
;
        CONST.D d4,0xFFFFFFF0           ; mask WDTHPW0,WDTLCK,ENDINIT
        and     d0,d4,d0                ; in copy of WDT_CON0 (d0)
        or      d0,d0,#0X03             ; WDTHPW0=0,WDTLCK=1,ENDINIT=1
        isync
        st16.w  [a3],d0                 ; set WDT_CON0
;
        ret                             ;
;--------------------------------------------------------------------------
